/*
  From http://paulbourke.net/dataformats/bmp/
  Modified for UnixPC

  C console program
 
  A plain black and white bitmap requires only a single bit to document each pixel.
  the bitmap is monochrome, and the palette contains two entries.
  each bit in the bitmap array represents a pixel. if the bit is clear,
  the pixel is displayed with the color of the first entry in the palette;
  if the bit is set, the pixel has the color of the second entry in the table.
  This type of bitmap has a very small file size, and doesn't require a palette.
 
  based on program bmpsuite.c by Jason Summers
  http://entropymine.com/jason/bmpsuite/
 
  Adam Majewski
  fraktal.republika.pl
  
*/       


#include <stdio.h>
#include <math.h>

unsigned short bswap16(unsigned short a)
{
  a = ((a & 0x00FF) << 8) | ((a & 0xFF00) >> 8);
  return a;
}

unsigned int bswap32(unsigned int a)
{
  a = ((a & 0x000000FF) << 24) |
      ((a & 0x0000FF00) <<  8) |
      ((a & 0x00FF0000) >>  8) |
      ((a & 0xFF000000) >> 24);
  return a;
}

                   /* width, height in pixels */
int pf1bit_bmp(char *fn, unsigned long Width, unsigned long Height,
   unsigned char bytes[]) {
   
    static unsigned char byte[1]; /* 1 byte = 8 bits */
    unsigned short word[1]; /* 2 bytes */
    unsigned long  dword[1], /* 4 bytes */
                   bpp=1,
                   /* in bytes */
                   FileHeaderSize=14,
                   InfoHeaderSize=40, /* header_bytes */
                   PaletteSize=(1<<bpp)*4, /* = 8 = number of bytes in palette */
                   BytesPerRow,
		   BytesPadRow,
                   FileSize,
                   OffBits,
		   Row,
                   BytesSize; /* bytes in image portion */
    FILE *fp;

    /*----------------------- */
    BytesPerRow=(((Width * bpp)+31)/32)*4;
    BytesPadRow=BytesPerRow - Width*bpp/8;
    BytesSize=BytesPerRow*Height;
    FileSize=FileHeaderSize+InfoHeaderSize+PaletteSize+BytesSize;
    OffBits=FileHeaderSize+InfoHeaderSize+PaletteSize;

    /*-------------------------- */
    fp = fopen(fn, "wb"); /* b - binary mode */
    if (fp == NULL) {
       perror("Unable to open output file");
       exit(1);
    }
#if 0
    printf("BytesPerRow= %d\n", BytesPerRow);
    printf("BytesPadRow= %d\n", BytesPadRow);
    printf("BytesSize= %d\n", BytesSize);
    printf("FileSize= %d\n", FileSize);
    printf("OffBits= %d\n", OffBits);
#endif

    /* bmp file header */
    word[0]=bswap16(19778); fwrite(word,1,2,fp); /* file Type signature = BM */
    dword[0]=bswap32(FileSize);   fwrite(dword,1,4,fp); /* FileSize */
    word[0]=0;           fwrite(word,1,2,fp); /* reserved1 */
    word[0]=0;           fwrite(word,1,2,fp); /* reserved2 */
    dword[0]=bswap32(OffBits);    fwrite(dword,1,4,fp); /* OffBits */
    dword[0]=bswap32(InfoHeaderSize);   fwrite(dword,1,4,fp);
    dword[0]=bswap32(Width);            fwrite(dword,1,4,fp);
    dword[0]=bswap32(Height);           fwrite(dword,1,4,fp);
    word[0]=bswap16(1);                 fwrite(word,1,2,fp); /* planes */
    word[0]=bswap16(1);           fwrite(word,1,2,fp); /* Bits of color per pixel */
    dword[0]=0;          fwrite(dword,1,4,fp); /* compression type */
    dword[0]=0;
    fwrite(dword,1,4,fp); /* Image Data Size, set to 0 when no compression */
    dword[0]=0;          fwrite(dword,1,4,fp); /*  */
    dword[0]=0;          fwrite(dword,1,4,fp); /*  */
    dword[0]=bswap32(2); fwrite(dword,1,4,fp); /*  number of used coloors*/
    dword[0]=0;          fwrite(dword,1,4,fp); /*  */
          
    /*  color table (palette) = 2 colors as a RGBA */
    /* color 0 = black */
    byte[0]=0;           fwrite(byte,1,1,fp); /* R */
    byte[0]=0;           fwrite(byte,1,1,fp); /* G */   
    byte[0]=0;           fwrite(byte,1,1,fp); /* B */
    byte[0]=255;         fwrite(byte,1,1,fp); /* A*/ 
    /* color 1 = green RGB from picture of screen */  
    byte[0]=44;          fwrite(byte,1,1,fp); /* R */
    byte[0]=255;         fwrite(byte,1,1,fp); /* G */
    byte[0]=120;         fwrite(byte,1,1,fp); /* B */
    byte[0]=255;         fwrite(byte,1,1,fp); /* A */ 
    dword[0] = 0;
    /* pixel color  data, input is unpadded. Add padding if needed  */
    for (Row = 0; Row < Height; Row++) {
       fwrite(&bytes[Row * (BytesPerRow - BytesPadRow)], 1, 
	  BytesPerRow - BytesPadRow, fp); 
       fwrite(dword, 1, BytesPadRow,fp);
    }
    fclose(fp);
    /*printf("file saved\n");*/
    return 0;
 }


